
## The power analysis was ran as follows: The number of effect SNPs was varied (10, 30, 100 or 300) as was the percentage of variance 
## explained by the effect SNPs (1, 2, 3, 4, 5 or 10).

## For each combination the requisite number of SNPs was randomly selected, 
## then effect sizes were drawn from a standard normal distribution and assigned them to those SNPs. 
## The predicted phenotype for each individual in each sample was subsequently computed using those effect sizes. 
## 1000 phenotype vectors were then generated (separately for each sample) by adding normally 
## distributed noise with variance such that the predicted phenotype accounted for the requisite percentage of the total 
## phenotypic (population) variance. For each of these 1000 phenotypes PLINK was used and the gene-set p-values were derived using JAG. 
## Stouffer's method was then used to combine the gene-set p-values across samples, and finally computed the power as the percentage of those 1000 in which the p-value < 0.05.
## This method was repeated 100 times for each combination of the number of effect SNPs  and the percentage of variance that they expalained  (ie. New effect sizes were drawn 100 times). 





flags = commandArgs(T)

# Settings
no.effects = 10       # number of SNPs with non-zero effect
var.explained = 0.05  # proportion of phenotypic variance explained by genetic component
no.sims = 100          # number of simulated phenotypes to generate (for each data set) per snp.effect vector
no.repeats = 10      # number of snp.effect vectors to generate

out.dir = ""          # directory to put output files into (or "" if not applicable)
out.suffix = "sim1"   # suffix to put on output file name for this simulation

if (length(flags) > 0) out.suffix = flags[1]
if (length(flags) > 1) no.effects = as.numeric(flags[2])
if (length(flags) > 2) var.explained = as.numeric(flags[3])/100
if (length(flags) > 3) no.sims = as.numeric(flags[4])
if (length(flags) > 4) no.repeats = as.numeric(flags[5])


prefixes = paste(c("abc", "lbc21", "lbc36", "man", "new"), sep="") # filename prefixes for each data-set used in the current paper
no.sets = length(prefixes)

# Load genotype data for relevant SNPs into R (per data-set)
geno = list()
snps = list()
fam = list()
snp.names = c()
for (i in 1:no.sets) {
  pref = prefixes[i]

  # Load SNP data (0/1/2 coding) into dat
  geno[[i]] = read.table(paste(pref, ".dat", sep=""), header=T, stringsAsFactors=F)

  # Set corresponding vector of SNP IDs
  snps[[i]] = names(geno[[i]])
  snp.names = c(snp.names, snps[[i]])
  
  # Read first two columns of ped/fam file (need the FID and IID values for this data-set to put into pheno file)
  fam[[i]] = read.table(paste(pref, ".ids", sep=""), header=T, stringsAsFactors=F)

  # For simplicity, do mean imputation on missing SNPs
  for (j in 1:dim(geno[[i]])[2]) {
    miss = is.na(geno[[i]][,j])
    if (sum(miss) > 0) geno[[i]][miss,j] = mean(geno[[i]][,j], na.rm=T)
  }
  
  # Standardize the SNP data -> compensates for overall MAF differences between data-sets, effect sizes will vary between data-sets otherwise
  geno[[i]] = sweep(sweep(geno[[i]], 2, apply(geno[[i]], 2, mean), FUN="-"), 2, apply(geno[[i]], 2, sd), FUN="/")
}
snp.names = unique(snp.names)

no.snps = length(snp.names)
sample.sizes = unlist(lapply(geno, function(x){dim(x)[1]}))

if (out.dir != "") out.dir = paste(out.dir, "/", sep="")
for (rep.no in 1:no.repeats) {
  # Simulate coefficients for each SNP -> generate coefficients from a standard normal distribution
  # NOTE: there is a chance the script will fail, if for some data-set the non-zero SNP effects all happen to be assigned to SNPs not present in that particular data set (gen.effect variance will be zero)
  # For the most part this is virtually guaranteed not to happen, but might become an issue for very small values of 'no.effects' (ie. 1 or 2)
  snp.effect = rep(0, no.snps)
  snp.effect[sample(no.snps, no.effects)] = rnorm(no.effects)


  # Determine 'true' genetic effect for each data set
  # Raw SNP effect-sizes (after compensating for MAF differences, see standardization above) are the same across data sets
  # However, due to differences in LD and in which SNPs are actually present in a given data set, the true effect sizes will vary somewhat
  # Proportion of variance explained is identical across data-sets, though
  gen.effect = list()
  for (i in 1:no.sets) {
    # Fetch relevant SNP effects for this data set, in correct order
    curr.effect = snp.effect[match(snps[[i]], snp.names)]
    gen.effect[[i]] = apply(sweep(geno[[i]], 2, curr.effect, FUN="*"), 1, sum)

    # Rescale the variance such that adding standard normal noise later yields the proper amount of explained variance
    # With G the variance of the genetic effect and E the noise variance, var.explained = G / (G + E). Therefore, setting E = 1, G = var.explained / (1 - var.explained)
    G = var.explained / (1-var.explained)
    curr.var = var(gen.effect[[i]]) * (sample.sizes[i]-1) / sample.sizes[i] # var() divides by N-1, but need division by N here
    gen.effect[[i]] = (gen.effect[[i]] - mean(gen.effect[[i]])) * sqrt(G / curr.var)
  }


  # Generate phenotype file with simulated phenotypes, to be used with plink --all-pheno flag
  for (i in 1:no.sets) {
    pref = prefixes[i]

    # Generate simulated phenotypes
    sims = sweep(matrix(rnorm(sample.sizes[i] * no.sims), ncol=no.sims), 1, gen.effect[[i]], FUN="+")

    sims = cbind(fam[[i]], sims)
    names(sims)[-(1:2)] = paste("Y", 1:no.sims, sep="")

    # Write to file
    write.table(sims, file=paste(out.dir, pref, "_", out.suffix, "_rep", rep.no, ".pheno", sep=""), row.names=F, quote=F, sep="\t")
  }
}




